import 'package:json_annotation/json_annotation.dart';

import 'IDetailMenu.dart' show IDetailMenu;
import 'IDetailPrepa.dart' show IDetailPrepa;
import 'IDetailStock.dart' show IDetailStock;
import "IR5BtnProperties.dart" show IR5BtnProperties;

part 'IR5Article.g.dart';
// --------------------------------------------------------------------------------

/** After Pointage Auto Goto ... */
enum GotoMode {
  @JsonValue(-10)
  GotoPreparation,
  @JsonValue(-20)
  GotoDepartement,
  @JsonValue(1)
  GotoOnglet
}
// --------------------------------------------------------------------------------

/** Articles */
@JsonSerializable(explicitToJson: true)
class IR5Article {
  IR5Article({
    required this.PLU,
    required this.Categorie,
    required this.Nom,
    required this.Type,
    this.TypeDescr,
    required this.Flags,
    required this.Allergens,
    required this.Calories,
    required this.Action,
    required this.NomAddLg1,
    required this.NomAddLg2,
    required this.NomAddLg3,
    required this.NomProd,
    required this.BtnText,
    required this.BtnProp,
    required this.MemoCuisine,
    required this.Description,
    required this.ImageInfo,
    required this.ExtRef,
    required this.Tags,
    required this.WebOrder,
    required this.NomWebLg1,
    required this.NomWebLg2,
    required this.NomWebLg3,
    required this.WebDescrLg1,
    required this.WebDescrLg2,
    required this.WebDescrLg3,
    required this.Barcode,
    required this.Desti,
    required this.gotoMode,
    required this.GotoOnglet,
    required this.CategPos,
    required this.Groupe,
    required this.Divisions,
    required this.TVA,
    required this.TVACoeff,
    required this.TVAEmport,
    required this.TVAEmportCoeff,
    required this.PrixAchat,
    required this.PrixAchatEmport,
    required this.Prix1,
    required this.Prix2,
    required this.Prix3,
    required this.Prix4,
    required this.Prix1HappyHour,
    required this.Prix2HappyHour,
    required this.Prix3HappyHour,
    required this.Prix4HappyHour,
    required this.Unit,
    required this.Packaging,
    required this.Tare,
    required this.PrimeGarcon,
    required this.DetailsAuto,
    required this.DetailsPrepa,
    required this.DetailsMenu,
    required this.DetailsStock,
    required this.CreationDate,
    required this.LastModifDate,
    required this.LastModifStaff,
    this.Bloque,
    this.Pointe,
    this.Disabled,
    this.PrixASaissir,
    this.NomASaissir,
    this.OnlyResto,
    this.OnlyEmport,
    this.ForceEmport,
    this.PointageNoCumul,
    this.ForceResto,
    this.InterditAZero,
    this.InterditHorsMenu,
    this.InterditDansMenu,
    this.OnlyOnceRemise,
    this.Alcool,
    this.HappyHourQty,
    this.NotInTotalDiscount,
    this.NoChgPrix,
    this.NonFood,
    this.ControlINOUT,
    this.MenuForfaitCandidat,
    this.KeepStock,
    this.ListeReassort,
    this.BlocageAuto,
    this.AuCentimetre,
    this.NoDetailsIfResto,
    this.NoDetailsIfEmport,
    this.OnlyManager,
    this.Balance,
    this.External,
    this.AdditionNoCumul,
    this.NoTicketIfEmport,
    this.Deluxe,
    this.IsStockIngredient,
    this.IsStockCarton,
    this.StayInDepartement,
    this.ForceDetailLinked,
    this.PreparationOpen,
    this.PreparationControled,
    this.PreparationGrouped,
    this.PreparationShowNivDescr,
    this.PreparationSortByNom,
    this.NeedDrapeau4Lancer,
    this.NeedDrapeau4Add,
    this.RptModifFlag1,
    this.RptModifFlag2,
    this.RptModifFlag3,
    this.RptModifFlag4,
  });

  factory IR5Article.fromJson([Map<String, dynamic>? json]) =>
      _$IR5ArticleFromJson(json ?? {});
  Map<String, dynamic> toJson() => _$IR5ArticleToJson(this);

  /** Article Unique Code (Price Look-Up) */
  @JsonKey(defaultValue: '')
  final String PLU;

  /** Category to which the article belongs */
  @JsonKey(defaultValue: '')
  final String Categorie;

  /** Article Name (NomPLU/NomProd/NomAdd/...) */
  @JsonKey(defaultValue: '')
  final String Nom;

  /** Article Types
   *
   * 0 - Article Simple
   * 6 - Message Simple
   * 7 - Message Linked
   * 4 - Message Plus
   * 5 - Message Minus
   * 17 - Multiplicateur Quantite
   * 18 - Multiplicateur Discount
   * 19 - Multiplicateur Prix
   * 20 - Message Drapeau
   * 21 - Time Drapeau
   * 22 - TelNr Drapeau
   * 40 - Tête de Menu
   * 93 - Suite
   * 94 - Remise Sur Tot
   * 95 - Remise Sur Food
   * 96 - Remise Sur NonFood
   * 97 - Remise Sur Sel
   * 98 - Remise Abs
   * 99 - InternalAction
   */
  @JsonKey(defaultValue: 0)
  final int Type;

  final String? TypeDescr;

  /** Article Flags
   *   Bit 00 : Bloque
   *   Bit 01 : Pointe
   *   Bit 02 : Disabled
   *   Bit 03 : PrixASaissir
   *   Bit 04 : NomASaissir
   *   Bit 05 : OnlyResto
   *   Bit 06 : OnlyEmport
   *   Bit 07 : ForceEmport
   *   Bit 08 : PointageNoCumul
   *   Bit 09 : ForceResto
   *   Bit 10 : InterditAZero
   *   Bit 11 : InterditHorsMenu
   *   Bit 12 : InterditDansMenu
   *   Bit 13 : OnlyOnceRemise
   *   Bit 14 : Alcool
   *   Bit 15 : HappyHourQty
   *   Bit 16 : NotInTotalDiscount
   *   Bit 17 : NoChgPrix
   *   Bit 18 : NonFood
   *   Bit 19 : ControlINOUT
   *   Bit 20 : MenuForfaitCandidat
   *   Bit 21 : KeepStock
   *   Bit 22 : ListeReassort
   *   Bit 23 : BlocageAuto
   *   Bit 24 : AuCentimetre
   *   Bit 25 : NoDetailsIfResto
   *   Bit 26 : NoDetailsIfEmport
   *   Bit 27 : OnlyManager
   *   Bit 28 : Balance
   *   Bit 29 : External
   *   Bit 30 : AdditionNoCumul
   *   Bit 31 : NoTicketIfEmport
   *   Bit 32 : Deluxe
   *   Bit 33 : IsStockIngredient
   *   Bit 34 : IsStockCarton
   *   Bit 35 : StayInDepartement
   *   Bit 36 : ForceDetailLinked
   *   Bit 40 : PreparationOpen
   *   Bit 41 : PreparationControled
   *   Bit 42 : PreparationGrouped
   *   Bit 43 : PreparationShowNivDescr
   *   Bit 44 : PreparationSortByNom
   *   Bit 45 : NeedDrapeau4Lancer
   *   Bit 46 : NeedDrapeau4Add
   *   Bit 51 : RptModifFlag1
   *   Bit 52 : RptModifFlag2
   *   Bit 53 : RptModifFlag3
   *   Bit 54 : RptModifFlag4
   */
  @JsonKey(defaultValue: 0)
  final int Flags;

  /** Allergens
   *   Bit 00 : Céréales
   *   Bit 01 : Crustacés
   *   Bit 02 : OEufs
   *   Bit 03 : Poissons
   *   Bit 04 : Arachides
   *   Bit 05 : Soja
   *   Bit 06 : Lait
   *   Bit 07 : Fruits à coque
   *   Bit 08 : Céleri
   *   Bit 09 : Moutarde
   *   Bit 10 : Graines de sésame
   *   Bit 11 : Anhydride sulfureux et sulfites
   *   Bit 12 : Lupin
   *   Bit 13 : Mollusques
   */
  @JsonKey(defaultValue: 0)
  final int Allergens;

  /** Cal. pour 100 g */
  @JsonKey(defaultValue: 0)
  final int Calories;
  /*
  @JsonKey(defaultValue: HistoAction.Article)
  final HistoAction? Action;
  */
  @JsonKey(defaultValue: '')
  final String Action;
  /** Article Name for Bills & Invoices. Coalesce to NomPLU */
  @JsonKey(defaultValue: '')
  final String NomAddLg1;

  /** Article Name for Bills & Invoices. Coalesce to NomPLU */
  @JsonKey(defaultValue: '')
  final String NomAddLg2;

  /** Article Name for Bills & Invoices. Coalesce to NomPLU */
  @JsonKey(defaultValue: '')
  final String NomAddLg3;

  /** Article Name for Tickets Productions (Kitchen/Bar/...). Coalesce to NomPLU. For all Languages, not only FR */
  @JsonKey(defaultValue: '')
  final String NomProd;

  /** Article Name for Screen Buttons. Coalesce to NomPLU. For all Languages, not only FR */
  @JsonKey(defaultValue: '')
  final String BtnText;

  final IR5BtnProperties? BtnProp;

  /** Article MEMOCUISINE */
  @JsonKey(defaultValue: '')
  final String MemoCuisine;

  /** Just a free Description */
  @JsonKey(defaultValue: '')
  final String Description;

  /** Big Image Info */
  @JsonKey(defaultValue: 0)
  final int ImageInfo;

  /** External Ref (used as primary key) */
  @JsonKey(defaultValue: '')
  final String ExtRef;

  /** Free Tags or Labels semicolumn separated */
  @JsonKey(defaultValue: '')
  final String Tags;

  /** Export to WebOrder Platforms (semicolumn separated list of Platforms)
   * Export ModePay if ModePay.WebOrder
   * Export Article if Article.WebOrder or Article.Categ.WebOrder or Article.Group.WebOrder
   */
  @JsonKey(defaultValue: '')
  final String WebOrder;

  /** Article WebName in Lg1. Coalesce to NomAdd[Lg1]. In Import will also Set NomAdd[Lg] if NomAdd[Lg] Not Set */
  @JsonKey(defaultValue: '')
  final String NomWebLg1;

  /** Article WebName in Lg2. Coalesce to NomAdd[Lg2]. In Import will also Set NomAdd[Lg] if NomAdd[Lg] Not Set */
  @JsonKey(defaultValue: '')
  final String NomWebLg2;

  /** Article WebName in Lg3. Coalesce to NomAdd[Lg3]. In Import will also Set NomAdd[Lg] if NomAdd[Lg] Not Set */
  @JsonKey(defaultValue: '')
  final String NomWebLg3;

  /** Article Web Memo Description in Lg1. Coalesce to Description */
  @JsonKey(defaultValue: '')
  final String WebDescrLg1;

  /** Article Web Memo Description in Lg2. Coalesce to Description */
  @JsonKey(defaultValue: '')
  final String WebDescrLg2;

  /** Article Web Memo Description in Lg3. Coalesce to Description */
  @JsonKey(defaultValue: '')
  final String WebDescrLg3;

  /** Article Barcode */
  @JsonKey(defaultValue: '')
  final String Barcode;

  /** Printer Destination to which the article belongs */
  @JsonKey(defaultValue: 0)
  final int Desti;

  // --------------------------------------------------------------------------------

  /** After Pointage Auto Goto ... */
  /*
  @JsonKey(name: 'GotoMode', defaultValue: GotoMode.GotoOnglet)
  final GotoMode gotoMode;
  */
  @JsonKey(name: 'GotoMode')
  final GotoMode? gotoMode;

  /** Tab to which the article belongs */
  @JsonKey(defaultValue: 0)
  final int GotoOnglet;

  /** Article sort order in relation to the Category */
  @JsonKey(defaultValue: 0)
  final int CategPos;

  /** Group to which the article belongs */
  @JsonKey(defaultValue: 0)
  final int Groupe;

  /** Accessible Division(s) for the Station,Staff,Article,.. */
  @JsonKey(defaultValue: '')
  final String Divisions;

  /** VAT Rate level as Defined in Etab */
  @JsonKey(defaultValue: 0)
  final num TVA;

  /** Articles VAT Ratio
   * Ratio0;Ratio1;Ratio2;Ratio3
   * 0;0;35;65 -->  35% Ratio2 + 65% Ratio3
   * 0;100     --> 100% Ratio1
   * ;;;100    --> 100% Ratio3 same as TYPETVA=3
   */
  @JsonKey(defaultValue: '')
  final String TVACoeff;

  /** VAT Rate level as Defined in Etab */
  @JsonKey(defaultValue: 0)
  final num TVAEmport;

  /** Articles VAT Ratio
   * Ratio0;Ratio1;Ratio2;Ratio3
   * 0;0;35;65 -->  35% Ratio2 + 65% Ratio3
   * 0;100     --> 100% Ratio1
   * ;;;100    --> 100% Ratio3 same as TYPETVA=3
   */
  @JsonKey(defaultValue: '')
  final String TVAEmportCoeff;

  /** Article Purchasing/Cost Price w/o TAX */
  @JsonKey(defaultValue: 0)
  final num PrixAchat;

  /** Article Purchasing/Cost Price w/o TAX when RestoOut(Emport) */
  @JsonKey(defaultValue: 0)
  final num PrixAchatEmport;

  /** Article Selling Price 1 */
  @JsonKey(defaultValue: 0)
  final num Prix1;

  /** Article Selling Price 2 */
  @JsonKey(defaultValue: 0)
  final num Prix2;

  /** Article Selling Price 3 */
  @JsonKey(defaultValue: 0)
  final num Prix3;

  /** Article Selling Price 4 */
  @JsonKey(defaultValue: 0)
  final num Prix4;

  /** Article HappyHour Selling Price 1 OR Qt */
  @JsonKey(defaultValue: 0)
  final num Prix1HappyHour;

  /** Article HappyHour Selling Price 2 OR Qt */
  @JsonKey(defaultValue: 0)
  final num Prix2HappyHour;

  /** Article HappyHour Selling Price 3 OR Qt */
  @JsonKey(defaultValue: 0)
  final num Prix3HappyHour;

  /** Article HappyHour Selling Price 4 OR Qt */
  @JsonKey(defaultValue: 0)
  final num Prix4HappyHour;

  /** Article unit of measure (Kg,Ml,L,...) */
  @JsonKey(defaultValue: '')
  final String Unit;

  /** Units Article amount in a package */
  @JsonKey(defaultValue: 0)
  final num Packaging;

  /** Tare to remove when the Article is weighed */
  @JsonKey(defaultValue: 0)
  final num Tare;

  @JsonKey(defaultValue: 0)
  final num PrimeGarcon;

  /** Pointage auto */
  @JsonKey(defaultValue: [])
  final List<String> DetailsAuto;

  /** Details Prepa/Departement de l'article */
  @JsonKey(defaultValue: [])
  final List<IDetailPrepa> DetailsPrepa;

  /** Menu Composition de l'article */
  @JsonKey(defaultValue: [])
  final List<IDetailMenu> DetailsMenu;

  /** Carton Composition or Stock Ingredients de l'article */
  @JsonKey(defaultValue: [])
  final List<IDetailStock> DetailsStock;

  /** Date Creation */
  @JsonKey(defaultValue: '')
  final String CreationDate;

  /** Date Last Modif */
  @JsonKey(defaultValue: '')
  final String LastModifDate;

  /** Staff who did last modif */
  @JsonKey(defaultValue: 0)
  final int LastModifStaff;

  final bool? Bloque;
  final bool? Pointe;
  final bool? Disabled;
  final bool? PrixASaissir;
  final bool? NomASaissir;
  final bool? OnlyResto;
  final bool? OnlyEmport;
  final bool? ForceEmport;
  final bool? PointageNoCumul;
  final bool? ForceResto;
  final bool? InterditAZero;
  final bool? InterditHorsMenu;
  final bool? InterditDansMenu;
  final bool? OnlyOnceRemise;
  final bool? Alcool;
  final bool? HappyHourQty;
  final bool? NotInTotalDiscount;
  final bool? NoChgPrix;
  final bool? NonFood;
  final bool? ControlINOUT;
  final bool? MenuForfaitCandidat;
  final bool? KeepStock;
  final bool? ListeReassort;
  final bool? BlocageAuto;
  final bool? AuCentimetre;
  final bool? NoDetailsIfResto;
  final bool? NoDetailsIfEmport;
  final bool? OnlyManager;
  final bool? Balance;
  final bool? External;
  final bool? AdditionNoCumul;
  final bool? NoTicketIfEmport;
  final bool? Deluxe;
  final bool? IsStockIngredient;
  final bool? IsStockCarton;
  final bool? StayInDepartement;
  final bool? ForceDetailLinked;
  final bool? PreparationOpen;
  final bool? PreparationControled;
  final bool? PreparationGrouped;
  final bool? PreparationShowNivDescr;
  final bool? PreparationSortByNom;
  final bool? NeedDrapeau4Lancer;
  final bool? NeedDrapeau4Add;
  final bool? RptModifFlag1;
  final bool? RptModifFlag2;
  final bool? RptModifFlag3;
  final bool? RptModifFlag4;
}
